%
%
% Unwarp function -  Usage and Description
% _____________________________________________________________________
%
% This manual describes the principles behind and the usage of the 
% "Realign and Unwarp" function in SPM. In particular it describes
% the joint use of the FieldMap toolbox and the Unwarp function.
%
% For help on how to use "Realign" (without Unwarp) see
% spm_realign_ui.m
%
% Unwarp - Usage
% _____________________________________________________________________
%
% Unwarp is invoked by picking "Realign & Unwarp" from the "Realign"
% menu.
%
% Prompts explained
%__________________
%
% 1. Num subjects
%    Specify the number of subjects from which you have data which 
%    you want to realign and unwarp. Note that unlike for the
%    modelling step the distinction between subjects and sessions
%    is important.
%
% 2. Include pre-calculated field maps?
%    Answer yes to this if you have measured and pre-processed 
%    fieldmaps created with FieldMap or equivalent.
%    You can supply one fieldmap per subject or one per session
%    or anything in between. The way we expect it to be used is
%    as follows; 
%      For most of the cases it should be sufficient with one 
%    fieldmap per subject. Select that map for the first session 
%    and simply press "done" without selecting any map for the 
%    remaining sessions. This will cause the selected map to be
%    used for all sessions.
%      In some cases one may have to take the subject out of the
%    scanner between two sessions. When putting the subject back
%    shimming will typically be redone, which means that it will
%    be a good idea to also collect a new fieldmap. When analysing
%    these data specify the first fieldmap for the first session
%    and just press "done" for the remaining sessions with the same
%    shim parameters. Specify the second fieldmap for the first
%    session after change in shim and just press "done" for the
%    rest of sessions. This means that the first map will be 
%    used for all sessions up til the change and the second map
%    used for all sessions after the change.
%
%    N.B. ALL FIELDMAPS SHOULD BE COREGISTERED TO THE FIRST
%    SCAN OF THE FIRST SESSION. This holds true also for when
%    multiple fieldmaps are used for one subject. This is 
%    accomplished in FieldMap by i) create fieldmap, ii) load
%    first scan from first session ("Load EPI image" button)
%    and iii) Press "Match VDM to EPI" button.
%
%    If you are using a software different from FieldMap note 
%    that
%    a) These should really be voxel-displacement maps, i.e.
%       scaled such that the values in the map specify how far
%       the sampling points of each voxel should be translated
%       in the phase-encode direction.
%    b) Take note to change the sign of the displacement map
%       depending on if your EPI images have been acquired with
%       positive or negative phase-encode blips.
%    c) It should be an analyse file in the same space as the 
%       first EPI image of the first session.
%
% 3. Num sessions, subj i
%    Specify number of sessions for subject # i.
%
% 4. Images, subj i, sess j
%    Pick images for session j in subject i.
%
% 5. FieldMap, subj i, sess j
%    Pick voxel displacement map (will have prefix 'vdm_' if
%    the FieldMap toolbox has been used) for session j in 
%    subject i, or press return without selecting anything.
%    If no fieldmap is selected the last fieldmap selected
%    for subject i will be used. A fieldmap MUST be selected
%    for the first session.
%
% 6. Model field changes w.r.t...
%    Theoretically (ignoring effects of shimming) one would
%    expect the field to depend only on subject out-of-plane
%    rotations. Hence the default choice ("Pitch and Roll").
%    Go with that unless you have very good reasons to do 
%    otherwise (see "Description" part of the document for
%    theory).
%
% 7. Create what
%    "Realign and Unwarp" will create a new set of images
%    prefixed by a 'u' (c.f. 'r' for "Realign"). One can
%    chose to also get a mean (averaged across all scans)
%    realigned and unwarped image. Useful primarily if one
%    plans to perform spatial normalisation directly on
%    the EPI images. There is no option to "Realign &
%    Unwarp only" (c.f. "Realign only") since the Unwarp
%    and the Spatial Normalisation doesn't coexist on
%    the parameter level.
%
% Defaults explained
%___________________
%
% 1. Filter width (mm)
%    FWHM of Gaussian filter applied to data prior to estimation 
%    of field.
%
% 2. No. of basis functions
%    No. of (in-plane) basis-functions. This parameter determines 
%    the spatial resolution by which we can estimate the 
%    rate-of-change fields. The no. of  basis-functions in the 
%    third direction will be determined by the image volume 
%    dimensions such that the resolution is as close to isotropic 
%    as possible.
%
% 3. Amount of regularisation
%    Unwarps looks for the solution that maximises the likelihood 
%    (minimises the variance) while simultaneously maximising the
%    smoothness of the estimated field (c.f. Lagrange multipliers).
%    This parameter determines how to balance the compromise 
%    between these (i.e. the value of the multiplier). Test it
%    on your own data (if you can be bothered) or go with the
%    defaults.
% 4. Re-estimation of movement parameters.
%    In addition to inducing residual (after realignment) movement-
%    related variance movement-by-susceptibility distortion changes
%    may bias the estimation of the movement. For each iteration 
%    Unwarp gets a better idea of the true shape of each scan, and
%    can potentially get a better estimate of movement. If answering
%    yes to this Unwarp will re-estimate (do a new realign) the
%    movements between each iteration of estimating the fields.
%    Our testing indicates that this is a good idea.
%
% 5. Include Jacobian intensity modulation.
%    See below for an explanation.
%    It turns out NOT to be a good idea.
%
% 6. Second order effects
%    Again, see below for an explanation. For "regular" use you
%    should probably not fiddle with this option. Could be fun to
%    try for those interested in the method per se though.
%
%     
% Unwarp - Description
% _____________________________________________________________________
%
% The ideas behind Unwarp are described in
%
% Andersson JLR, Hutton C, Ashburner J, Turner R, Friston K.
% 2001. Modelling geometric deformations in EPI time series.
% NeuroImage 13:903-919.
%
% The paper is unfortunately a bit old now and describes none of
% the newer features. Hopefully we'll have a second paper out any
% decade now.
%
% See also spm_uw_estimate.m for a detailed description of the
% implementation.
%
% Even after realignment there is considerable variance in fMRI time 
% series that covary with, and is most probably caused by, subject 
% movements. It is also the case that this variance is typically 
% large compared to experimentally induced variance. Anyone interested 
% can include the estimated movement parameters as covariates in the 
% design matrix, and take a look at an F-contrast encompassing those 
% columns. It is quite dramatic. The result is loss of sensitivity, 
% and if movements are correlated to task specificity. I.e. we may 
% mistake movement induced variance for true activations.
%
% The problem is well known, and several solutions have been suggested. 
% A quite pragmatic (and conservative) solution is to include the 
% estimated movement parameters (and possibly squared) as covariates 
% in the design matrix. Since we typically have loads of degrees of 
% freedom in fMRI we can usually afford this. The problems occur when 
% movements are correlated with the task, since the strategy above 
% will discard "good" and "bad" variance alike (i.e. remove also "true"
% activations.
%
% The "covariate" strategy described above was predicated on a model 
% where variance was assumed to be caused by "spin history" effects, 
% but will work pretty much equally good/bad regardless of what the 
% true underlying cause is.
%
% Others have assumed that the residual variance is caused mainly by 
% errors introduced by the interpolation kernel in the resampling step 
% of the realignment. One has tried to solve this through higher order 
% resampling (huge Sinc kernels, or k-space resampling).
%
% Unwarp is based on a different hypothesis regarding the residual 
% variance. EPI images are not particularly faithful reproductions of 
% the object, and in particular there are severe geometric distortions 
% in regions where there is an air-tissue interface (e.g. orbitofronal 
% cortex and the anterior medial temporal lobes). In these areas in 
% particular the observed image is a severely warped version of reality, 
% much like a funny mirror at a fair ground. When one moves in front of 
% such a mirror ones image will distort in different ways and ones head 
% may change from very elongated to seriously flattened. If we were to 
% take digital snapshots of the reflection at these different positions 
% it is rather obvious that realignment will not suffice to bring them 
% into a common space. 
%
% The situation is similar with EPI images, and an image collected for 
% a given subject position will not be identical to that collected at 
% another. We call this effect susceptibility-by-movement interaction. 
% Unwarp is predicated on the assumption that the susceptibility-by-
% movement interaction is responsible for a sizable part of residual 
% movement related variance.
%
% Assume that we know how the deformations change when the subject 
% changes position (i.e. we know the derivatives of the deformations 
% with respect to subject position). That means that for a given time 
% series and a given set of subject movements we should be able to 
% predict the "shape changes" in the object and the ensuing variance 
% in the time series. It also means that, in principle, we should be 
% able to formulate the inverse problem, i.e. given the observed 
% variance (after realignment) and known (estimated) movements we should 
% be able to estimate how deformations change with subject movement. 
%
% We have made an attempt at formulating such an inverse model, and at 
% solving for the "derivative fields". A deformation field can be 
% thought of as little vectors at each position in space showing how 
% that particular location has been deflected. A "derivative field" 
% is then the rate of change of those vectors with respect to subject 
% movement. Given these "derivative fields" we should be able to remove 
% the variance caused by the susceptibility-by-movement interaction. 
% Since the underlying model is so restricted we would also expect 
% experimentally induced variance to be preserved. Our experiments 
% have also shown this to be true. 
%
% In theory it should be possible to estimate also the "static" 
% deformation field, yielding an unwarped (to some true geometry) 
% version of the time series. In practise that doesn't really seem to 
% work. Hence, the method deals only with residual movement related 
% variance induced by the susceptibility-by-movement interaction.
%
% This means that the time-series will be undistorted to some
% "average distortion" state rather than to the true geometry.
% If one wants additionally to address the issue of anatomical
% fidelity one should combine Unwarp with a measured fieldmap. 
% 
% The description above can be thought of in terms of a Taylor 
% expansion of the field as a function of subject movement. Unwarp 
% alone will estimate the first (and optionally second, see below) 
% order terms of this expansion. It cannot estimate the zeroth 
% order term (the distortions common to all scans in the time 
% series) since that doesn't introduce (almost) any variance in
% the time series. The measured fieldmap takes the role of the 
% zeroth order term. Refer to the FieldMap toolbox and the 
% documents FieldMap.man and FieldMap_principles.man for a 
% description of how to obtain fieldmaps in the format expected
% by Unwarp.
% 
% If we think of the field as a function of subject movement it
% should in principle be a function of six variables since rigid
% body movement has six degrees of freedom. However, the physics
% of the problem tells us that the field should not depend on
% translations nor on rotation in a plane perpendicular to the
% magnetic flux. Hence it should in principle be sufficient to
% model the field as a function of out-of-plane rotations (i.e.
% pitch and roll). One can object to this in terms of the effects
% of shimming (object no longer immersed in a homogenous field)
% that introduces a dependence on all movement parameters. In 
% addition SPM/Unwarp cannot really tell if the transversal 
% slices it is being passed are really perpendicular to the flux
% or not. In practice it turns out thought that it is never (at
% least we haven't seen any case) necessarry to include more
% than Pitch and Roll. This is probably because the individual
% movement parameters are typically highly correlated anyway,
% which in turn is probably because most heads that we scan 
% are attached to a neck around which rotations occurr.
%
% On the subject of Taylor expansion we should mention that there
% is the option to use a second-order expansion (through the 
% defaults) interface. This implies estimating also the 
% rate-of-change w.r.t. to some movement parameter of 
% the rate-of-change of the field w.r.t. some movement parameter
% (colloquially known as a second derivative). It can be quite
% intresting to watch (and it is amazing that it is possible)
% but rarely helpful/necessarry.
%
% In the defaults there is also an option to include Jacobian
% intensity modulation when estimating the fields. "Jacobian
% intensity modulation" refers to the dilution/concentration
% of intensity that ensue as a consequence of the distortions.
% Think of a semi-transparent coloured rubber sheet that you
% hold against a white background. If you stretch a part of
% the sheet (induce distortions) you will see the colour fading
% in that particular area. In theory it is a brilliant idea to
% include also these effects when estimating the field (see e.g.
% Andersson et al, NeuroImage 20:870-888). In practice for this
% specific problem it is NOT a good idea.
% 
% It should be noted that this is a method intended to correct 
% data afflicted by a particular problem. If there is little 
% movement in your data to begin with this method will do you little 
% good. If on the other hand there is appreciable movement in your 
% data (>1deg) it will remove some of that unwanted variance. If, 
% in addition, movements are task related it will do so without 
% removing all your "true" activations. 
%
% The method attempts to minimise total (across the image volume) 
% variance in the data set. It should be realised that while 
% (for small movements) a rather limited portion of the total 
% variance is removed, the susceptibility-by-movement interaction 
% effects are quite localised to "problem" areas. Hence, for a 
% subset of voxels in e.g. frontal-medial and orbitofrontal cortices 
% and parts of the temporal lobes the reduction can be quite dramatic 
% (>90%).
%
% The advantages of using Unwarp will also depend strongly on the
% specifics of the scanner and sequence by which your data has been
% acquired. When using the latest generation scanners distortions
% are typically quite small, and distortion-by-movement interactions
% consequently even smaller. A small check list in terms of
% distortions is
%
% a) Fast gradients->short read-out time->small distortions
% b) Low field (i.e. <3T)->small field changes->small distortions
% c) Low res (64x64)->short read-out time->small distortions
% d) SENSE/SMASH->short read-out time->small distortions
%
% If you can tick off all points above chances are you have minimal
% distortions to begin with and you can say "sod Unwarp" (but not
% to our faces!).
%
% _____________________________________________________________________
% Good Luck! Jesper Andersson & Chloe Hutton


